// Show z-order
function(pageItem, doc, appVersion, cache)
{
//
// Plug-In for FrameReporter (http://www.rorohiko.com/framereporter). 
// Install in a subfolder of the 'Labels' folder next to the FrameReporter.spln 
// (Don't copy into 'Labels' - instead it needs to go in one of the available 
// subfolders, like 'TopBlue' or suchlike).
//
// Version 1.0.1
//
// (c) 2013 Rorohiko Ltd. 
//
// Written by Kris Coppieters, kris@rorohiko.com
//

    var err;
    var pageItemA;
    var pageItemB;
    var retVal;

    function getPageItemZOrderDict(container)
    {
        var retVal = {};

        var pageItems = container.allPageItems.slice(0);
        var pageItemReferences = [];
        for (var pageItemIdx = 0; pageItemIdx < pageItems.length; pageItemIdx++)
        {                    
            var pageItem = pageItems[pageItemIdx];
            try
            {
                pageItem.id;
                pageItemReferences.push({
                    item: pageItem,
                    index: pageItemIdx,
                    nextChildZIndex: 0,
                    zIndexPrefix: ""
                });
            }
            catch (err)
            {
            }
        }

        pageItemReferences.sort(sortInZOrderCallBack);
        
        var containerReference = 
        {
            item: container,
            index: -1,
            nextChildZIndex: 0,
            zIndexPrefix: ""
        };
    
        pageItemReferences.splice(0,0,containerReference);

       var stack = [ containerReference ];
    
        for (var pageItemIdx = 1 /* Skip container reference */; pageItemIdx < pageItemReferences.length; pageItemIdx++)
        {
            var pageItemReference = pageItemReferences[pageItemIdx];
            var pageItem = pageItemReference.item;
            var parent = getLayoutParent(pageItem);
            var stackIdx = stack.length;
            while (--stackIdx >= 0)
            {
                var parentReference = stack[stackIdx];
                if (parentReference.item == parent)
                {
                    break;
                }
            }
        
            if (stackIdx >= 0)
            {
                parentReference.nextChildZIndex++;
                pageItemReference.parentReference = parentReference;
                stack.slice(stackIdx);
                stack.push(pageItemReference);
            }
        }

        for (var pageItemIdx = 1; pageItemIdx < pageItemReferences.length; pageItemIdx++)
        {
            var pageItemReference = pageItemReferences[pageItemIdx];
            var pageItem = pageItemReference.item;
            var parentReference = pageItemReference.parentReference;
            var zIndex = parentReference.nextChildZIndex--;
            zIndex = parentReference.zIndexPrefix + zIndex;
            retVal[pageItem.id] = zIndex;
            pageItemReference.zIndexPrefix = zIndex + "-";
        }
       return retVal;
    }

    function getLayoutParent(pageItem)
    {
        var parent = null;
        try 
        {
            parent = pageItem.parent;
            if (parent instanceof Character)
            {
                if (gAppVersion < 4)
                {
                    parent = parent.parentTextFrame;
                }
                else
                {
                    parent = parent.parentTextFrames[0];
                }
            }
            else if (parent instanceof Page)
            {
                parent = parent.parent;
            }
        }
        catch (err) 
        {
            parent = null;
        }
        
        return parent;
    }

    function getParentSpread(pageItem)
    {
        var parent = null;
        try 
        {
            if (pageItem instanceof Spread || pageItem instanceof MasterSpread)
            {
                return pageItem;
            }
        
            var prvParent = pageItem;
            parent = getLayoutParent(pageItem);
            while (parent != null && parent != prvParent && ! (parent instanceof Spread) && ! (parent instanceof MasterSpread) && parent != prvParent) 
            {
                prvParent = parent;
                parent = getLayoutParent(parent);
            }
            if (! (parent instanceof Spread) && ! (parent instanceof MasterSpread)) 
            {
                parent = null;
            }
        }
        catch (err) 
        {
            parent = null;
        }
        
        return parent;
    }

    function sortInZOrderCallBack(pageItemReferenceA, pageItemReferenceB)
    {
        if (pageItemReferenceA == pageItemReferenceB)
        {
            return 0;
        }

        pageItemA = pageItemReferenceA.item;
        pageItemB = pageItemReferenceB.item;
        if (pageItemA == pageItemB)
        {
            return 0;
        }

        retVal = pageItemA.itemLayer.index - pageItemB.itemLayer.index;
        if (retVal != 0)
        {
            return retVal;
        }

        return pageItemReferenceA.index - pageItemReferenceB.index;
    }

    var zOrder = 0;
    try
    {
        var parentSpread = getParentSpread(pageItem);
        if (parentSpread != null)
        {
            var zOrderDict = null;
            if (cache != undefined)
            {
                zOrderDict = cache[parentSpread.id];
            }
            if (zOrderDict == null)
            {
                zOrderDict = getPageItemZOrderDict(parentSpread);
                if (cache != undefined)
                {
                    cache[parentSpread.id] = zOrderDict;
                }
            }
            var zOrder = zOrderDict[pageItem.id];
        }
    }
    catch (err)
    {
    }


    return zOrder;
}
